/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.WebConfiguration.*;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;

import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.io.Resource;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestClient;
import org.springframework.web.client.RestClientException;

import de.ozgcloud.antragsraum.WebConfiguration;
import de.ozgcloud.antragsraum.common.RestClientUtils;
import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.proxy.AntragraumGrpcGetAttachmentContentRequest;
import de.ozgcloud.antragsraum.proxy.AntragraumGrpcGetAttachmentMetadataRequest;
import de.ozgcloud.antragsraum.proxy.AntragraumGrpcGetAttachmentMetadataResponse;
import de.ozgcloud.antragsraum.security.AuthenticationHelper;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
class FileRestClient {

	static final String FIND_FILE_METADATA_API = "/api/v1/file/metadata";
	static final String GET_FILE_CONTENT_API = "/api/v1/file/content";
	static final String UPLOAD_FILE_API = "/api/v1/file";
	public static final String FILE_KEY = "file";
	public static final String VORGANG_ID_KEY = "vorgangId";
	static final String NO_FILE_ID_RECEIVED_MESSAGE = "No fileId received";

	private final RestClient restClient;

	public FileRestClient(@Qualifier(WebConfiguration.NACHRICHT_REST_CLIENT) RestClient restClient) {
		this.restClient = restClient;
	}

	AntragraumGrpcGetAttachmentMetadataResponse getFileMetadata(FileIdentificationData fileIdentificationData, final String address) {
		var request = new AntragraumGrpcGetAttachmentMetadataRequest();
		request.setFileId(fileIdentificationData.fileId());
		request.setNachrichtId(fileIdentificationData.nachrichtId());
		request.setSamlToken(AuthenticationHelper.getSamlToken());

		try {
			return restClient
			  .post()
			  .uri(FIND_FILE_METADATA_API)
			  .header(X_GRPC_ADDRESS, RestClientUtils.sanitizeAddress(address))
			  .contentType(MediaType.APPLICATION_JSON)
			  .body(request)
			  .accept(MediaType.APPLICATION_JSON)
			  .retrieve()
			  .toEntity(AntragraumGrpcGetAttachmentMetadataResponse.class).getBody();
		} catch (RestClientException e) {
			LOG.error("Error loading file metadata. {}", e.getMessage(), e);
			throw e;
		}
	}

	void uploadFile(final OzgUploadFile uploadFile, final String address, final CompletableFuture<String> fileIdFuture) {
		try {
			var response = sendFile(uploadFile, address);
			if (Objects.nonNull(response)) {
				fileIdFuture.complete(response);
			} else {
				fileIdFuture.completeExceptionally(new IllegalStateException(NO_FILE_ID_RECEIVED_MESSAGE));
			}
		} catch (Exception e) {
			fileIdFuture.completeExceptionally(e);
		}
	}

	private String sendFile(OzgUploadFile uploadFile, String address) {
		MultiValueMap<String, Object> requestBody = new LinkedMultiValueMap<>();
		requestBody.add(FILE_KEY, uploadFile.file().getResource());
		requestBody.add(VORGANG_ID_KEY, uploadFile.vorgangId());

		return restClient
		  .post()
		  .uri(UPLOAD_FILE_API)
		  .header(X_GRPC_ADDRESS, RestClientUtils.sanitizeAddress(address))
		  .contentType(MediaType.MULTIPART_FORM_DATA)
		  .body(requestBody)
		  .retrieve().toEntity(String.class).getBody();
	}

	void downloadFileContent(FileIdentificationData fileIdentificationData, final String address, final OutputStream out) {
		var request = new AntragraumGrpcGetAttachmentContentRequest();
		request.setFileId(fileIdentificationData.fileId());
		request.setNachrichtId(fileIdentificationData.nachrichtId());
		request.setSamlToken(AuthenticationHelper.getSamlToken());

		try {
			var fileContent = Optional.ofNullable(restClient
			  .post()
			  .uri(GET_FILE_CONTENT_API, fileIdentificationData.fileId())
			  .header(X_GRPC_ADDRESS, RestClientUtils.sanitizeAddress(address))
			  .contentType(MediaType.APPLICATION_JSON)
			  .body(request)
			  .accept(MediaType.APPLICATION_OCTET_STREAM)
			  .retrieve().toEntity(Resource.class).getBody());

			fileContent.ifPresent(file -> processFile(out, file));
		} catch (RestClientException e) {
			LOG.error("Error downloading file. {}", e.getMessage(), e);
			throw e;
		}
	}

	private static void processFile(final OutputStream out, Resource fileContent) {
		try {
			IOUtils.copy(fileContent.getInputStream(), out);
		} catch (IOException e) {
			LOG.error("Error writing file content to output", e);
			throw new TechnicalException(e);
		}
	}

}
