/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.attachments;

import java.io.OutputStream;
import java.util.concurrent.CompletableFuture;

import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import de.ozgcloud.antragsraum.common.VirusFoundException;
import de.ozgcloud.antragsraum.events.NachrichtEventService;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@RequiredArgsConstructor
public class FileService {

	private final @NonNull FileRemoteService fileRemoteService;
	private final @NonNull VirusScannerClient scanner;
	private final @NonNull NachrichtEventService nachrichtEventService;

	public OzgFile getFile(FileIdentificationData fileIdentificationData) {
		var nachrichtEvent = nachrichtEventService.getNachrichtEventById(fileIdentificationData.nachrichtEventId());

		return fileRemoteService.getFile(fileIdentificationData, nachrichtEvent.address());
	}

	void downloadFileContent(FileIdentificationData fileIdentificationData, OutputStream out) {
		var nachrichtEvent = nachrichtEventService.getNachrichtEventById(fileIdentificationData.nachrichtEventId());

		fileRemoteService.downloadFileContent(fileIdentificationData, nachrichtEvent.address(), out);
	}

	CompletableFuture<String> upload(String vorgangId, String nachrichtEventId, MultipartFile file) {
		var nachrichtEvent = nachrichtEventService.getNachrichtEventById(nachrichtEventId);

		return fileRemoteService.uploadFile(createUploadFile(vorgangId, file), nachrichtEvent.address());
	}

	OzgUploadFile createUploadFile(String vorgangId, MultipartFile file) throws VirusFoundException {
		if (isClean(file)) {
			LOG.info("File {} is clean, preparing upload", file.getOriginalFilename());

			return OzgUploadFile.builder()
			  .vorgangId(vorgangId)
			  .file(file)
			  .build();
		}

		LOG.error("Found viruses in file {}", file.getOriginalFilename());
		throw new VirusFoundException(vorgangId);
	}

	boolean isClean(MultipartFile file) {
		return scanner.scan(file).isEmpty();
	}

}
