/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import java.time.DateTimeException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;
import java.util.Objects;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import de.ozgcloud.antragsraum.attachments.FileIdentificationData;
import de.ozgcloud.antragsraum.attachments.FileService;
import de.ozgcloud.antragsraum.attachments.OzgFile;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcRueckfrage;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcRueckfrageAnswer;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@RequiredArgsConstructor
class NachrichtMapper {

	private final @NonNull FileService fileService;

	private List<OzgFile> getAttachments(List<String> fileIds, String nachrichtId, String nachrichtEventId) {
		if (Objects.isNull(fileIds)) {
			return List.of();
		}

		return fileIds.stream().filter(StringUtils::isNotEmpty).map(fileId -> getFile(fileId, nachrichtId, nachrichtEventId))
		  .filter(Objects::nonNull).toList();
	}

	private OzgFile getFile(String fileId, String nachrichtId, String nachrichtEventId) {
		var fileIdentificationData = FileIdentificationData.builder()
		  .fileId(fileId)
		  .nachrichtId(nachrichtId)
		  .nachrichtEventId(nachrichtEventId)
		  .build();

		return fileService.getFile(fileIdentificationData);
	}

	public Nachricht fromRestRueckfrage(final AntragraumproxyGrpcRueckfrage rueckfrage, final String id) {
		var builder = Nachricht.builder()
		  .vorgangId(rueckfrage.getVorgangId())
		  .id(rueckfrage.getId())
		  .nachrichtEventId(id)
		  .title(rueckfrage.getVorgangName())
		  .message(rueckfrage.getText())
		  .replyNachrichten(fromRestRueckfrageAnswer(rueckfrage, id))
		  .answeredAt(toMillisecondsTimestamp(rueckfrage.getAnsweredAt()))
		  .sendAt(toMillisecondsTimestamp(rueckfrage.getSentAt()))
		  .attachments(getAttachments(rueckfrage.getAttachmentFileId(), rueckfrage.getId(), id))
		  .accessible(Boolean.TRUE.equals(rueckfrage.getAccessible()))
		  .trustLevel(TrustLevelMapper.map(rueckfrage.getTrustLevel()))
		  .status(rueckfrage.getStatus());

		return builder.build();
	}

	private long toMillisecondsTimestamp(String dateString) {
		long timestamp = 0;
		try {
			timestamp = LocalDateTime.parse(dateString).toEpochSecond(ZoneOffset.UTC) * 1000;
		} catch (DateTimeException e) {
			if (StringUtils.isEmpty(dateString)) {
				LOG.debug("Empty date received for Nachricht. Setting it to 0 (=1970-01-01)");
			} else {
				LOG.warn("Invalid date [{}] received for Nachricht. Setting it to 0 (=1970-01-01)", dateString);
			}
		}

		return timestamp;
	}

	private List<ReplyNachricht> fromRestRueckfrageAnswer(final AntragraumproxyGrpcRueckfrage rueckfrage, final String id) {
		if (Objects.isNull(rueckfrage.getAnswers())) {
			return List.of();
		}
		return rueckfrage.getAnswers().stream()
		  .map(answer -> {
			  var builder = ReplyNachricht.builder()
				.id(answer.getRueckfrageId())
				.rueckfrageId(rueckfrage.getId())
				.nachrichtEventId(id)
				.message(answer.getAnswerText())
				.attachments(getAttachments(answer.getAttachmentFileId(), answer.getRueckfrageId(), id));
			  return builder.build();
		  }).toList();
	}

	public AntragraumproxyGrpcRueckfrageAnswer toRestRueckfrageAnswer(final ReplyNachricht nachricht) {
		var antwort = new AntragraumproxyGrpcRueckfrageAnswer();
		antwort.setRueckfrageId(nachricht.rueckfrageId());
		antwort.setAnswerText(nachricht.message());

		if (Objects.nonNull(nachricht.attachments())) {
			antwort.attachmentFileId(nachricht.attachments().stream()
			  .map(OzgFile::id).toList());
		} else {
			antwort.setAttachmentFileId(List.of());
		}

		return antwort;
	}

}
