/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import de.ozgcloud.antragsraum.command.CommandReference;
import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcSendRueckfrageAnswerResponse;
import de.ozgcloud.antragsraum.security.AuthenticationHelper;
import io.grpc.StatusRuntimeException;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
@RequiredArgsConstructor
class NachrichtenRemoteService {

	private final NachrichtenRestClient nachrichtenRestClient;
	private final NachrichtMapper nachrichtMapper;
	private final AntragraumProxyMapper antragraumProxyMapper;

	List<NachrichtHeader> findRueckfrageHeads(NachrichtEvent event) {
		try {
			var reply = nachrichtenRestClient.getFindRueckfragen(event.postfachId(), AuthenticationHelper.getSamlToken(), event.address());
			var rueckfrageHeads = reply.getRueckfrageHead();

			return Objects.nonNull(rueckfrageHeads) ? rueckfrageHeads.stream()
			  .map(rueckfrageHead -> NachrichtHeaderMapper.fromRueckfrageHead(rueckfrageHead, event.id())).toList() : List.of();
		} catch (StatusRuntimeException e) {
			LOG.error("Error beim Laden der RueckfrageHeads für das Postfach {}", event.postfachId(), e);
		}

		return List.of();
	}

	Optional<Nachricht> getRueckfrage(String rueckfrageId, NachrichtEvent event) {
		try {
			var reply = nachrichtenRestClient.getGetRueckfrage(rueckfrageId, AuthenticationHelper.getSamlToken(), event.address());
			var rueckfrage = reply.getRueckfrage();

			return Objects.nonNull(rueckfrage) ? Optional.of(nachrichtMapper.fromRestRueckfrage(rueckfrage, event.id())) : Optional.empty();
		} catch (StatusRuntimeException e) {
			LOG.error("Error beim Laden der Rueckfrage mit ID {}", rueckfrageId, e);
		}

		return Optional.empty();
	}

	CommandReference sendAnswer(ReplyNachricht nachricht, String address) {
		var rueckfrage = antragraumProxyMapper.toRestProxyAntwort(nachricht);
		AntragraumproxyGrpcSendRueckfrageAnswerResponse response = nachrichtenRestClient.answerRueckfrage(
		  rueckfrage,
		  AuthenticationHelper.getSamlToken(), address);

		return CommandReference.builder()
		  .id(response.getCommandId())
		  .nachrichtEventId(nachricht.nachrichtEventId())
		  .build();
	}

}
