/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static de.ozgcloud.antragsraum.WebConfiguration.*;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestClient;
import org.springframework.web.client.RestClientException;

import de.ozgcloud.antragsraum.WebConfiguration;
import de.ozgcloud.antragsraum.common.RestClientUtils;
import de.ozgcloud.antragsraum.proxy.AntragraumGrpcFindRueckfragenRequest;
import de.ozgcloud.antragsraum.proxy.AntragraumGrpcGetRueckfrageRequest;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcFindRueckfragenResponse;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcGetRueckfrageResponse;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcRueckfrageAnswer;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcSendRueckfrageAnswerRequest;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcSendRueckfrageAnswerResponse;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
public class NachrichtenRestClient {

	static final String GET_RUECKFRAGEN_URI = "api/v1/rueckfragen";
	static final String GET_RUECKFRAGE_URI = "api/v1/rueckfrage";
	static final String SEND_ANTWORT_URI = "api/v1/antworten";

	private final RestClient restClient;

	public NachrichtenRestClient(@Qualifier(WebConfiguration.NACHRICHT_REST_CLIENT) RestClient nachrichtRestClient) {
		this.restClient = nachrichtRestClient;
	}

	AntragraumproxyGrpcFindRueckfragenResponse getFindRueckfragen(String postfachId, String token, String address) {
		try {
			var request = new AntragraumGrpcFindRueckfragenRequest();
			request.setPostfachId(postfachId);
			request.setSamlToken(token);
			return restClient.post().uri(GET_RUECKFRAGEN_URI)
			  .contentType(MediaType.APPLICATION_JSON)
			  .accept(MediaType.APPLICATION_JSON)
			  .header(X_GRPC_ADDRESS, RestClientUtils.sanitizeAddress(address))
			  .body(request)
			  .retrieve()
			  .toEntity(AntragraumproxyGrpcFindRueckfragenResponse.class).getBody();

		} catch (RestClientException e) {
			LOG.error("Error loading RueckfragenHeaders. {}", e.getMessage(), e);
			throw e;
		}
	}

	AntragraumproxyGrpcGetRueckfrageResponse getGetRueckfrage(String id, String token, String address) {
		try {
			var request = new AntragraumGrpcGetRueckfrageRequest();
			request.setSamlToken(token);
			request.setId(id);

			return restClient.post().uri(GET_RUECKFRAGE_URI)
			  .contentType(MediaType.APPLICATION_JSON)
			  .accept(MediaType.APPLICATION_JSON)
			  .header(X_GRPC_ADDRESS, RestClientUtils.sanitizeAddress(address))
			  .body(request)
			  .retrieve()
			  .toEntity(AntragraumproxyGrpcGetRueckfrageResponse.class).getBody();

		} catch (RestClientException e) {
			LOG.error("Error loading Rueckfrage. {}", e.getMessage(), e);
			throw e;
		}
	}

	AntragraumproxyGrpcSendRueckfrageAnswerResponse answerRueckfrage(AntragraumproxyGrpcRueckfrageAnswer antwort, String token, String address) {
		AntragraumproxyGrpcSendRueckfrageAnswerRequest request = new AntragraumproxyGrpcSendRueckfrageAnswerRequest();
		request.setAnswer(antwort);
		request.setSamlToken(token);

		try {
			return restClient.post().uri(SEND_ANTWORT_URI)
			  .contentType(MediaType.APPLICATION_JSON).header(X_GRPC_ADDRESS, RestClientUtils.sanitizeAddress(address))
			  .body(request)
			  .retrieve()
			  .toEntity(AntragraumproxyGrpcSendRueckfrageAnswerResponse.class).getBody();

		} catch (RestClientException e) {
			LOG.error("Error loading Rueckfrage. {}", e.getMessage(), e);
			throw e;
		}
	}

}
