/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.security;

import java.time.ZonedDateTime;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.stereotype.Component;

import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
class JwtTokenProvider {

	public static final String TOKEN_TYPE = "JWT";
	public static final String TOKEN_ISSUER = "antragsraum-api";
	public static final String TOKEN_AUDIENCE = "antragsraum-app";

	@Value("${ozgcloud.jwt.secret}")
	private String jwtSecret;

	public String generate(User user) {
		List<String> roles = createAuthorities(user);

		byte[] signingKey = jwtSecret.getBytes();

		return createToken(user, signingKey, roles);
	}

	@NonNull
	private static List<String> createAuthorities(User user) {
		return user.getAuthorities()
		  .stream()
		  .map(GrantedAuthority::getAuthority)
		  .toList();
	}

	private String createToken(User user, byte[] signingKey, List<String> roles) {
		return Jwts.builder()
		  .header().add("typ", TOKEN_TYPE)
		  .and()
		  .signWith(Keys.hmacShaKeyFor(signingKey), Jwts.SIG.HS512)
		  .expiration(user.getTokenExpiresAt())
		  .issuedAt(Date.from(ZonedDateTime.now().toInstant()))
		  .id(UUID.randomUUID().toString())
		  .issuer(TOKEN_ISSUER)
		  .audience().add(TOKEN_AUDIENCE)
		  .and()
		  .subject(user.getUsername())
		  .claim("refreshcode", user.getRefreshCode())
		  .claim("rol", roles)
		  .claim("firstname", user.getFirstName())
		  .claim("lastname", user.getLastName())
		  .claim("postkorbhandle", user.getPostkorbHandle())
		  .claim("trustlevel", user.getTrustLevel())
		  .compact();
	}

}
