/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.security;

import java.security.SecureRandom;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.security.saml2.provider.service.authentication.DefaultSaml2AuthenticatedPrincipal;
import org.springframework.security.saml2.provider.service.authentication.Saml2Authentication;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class UserMapper {

	public static final String POSTKORB_HANDLE_URN = "urn:oid:2.5.4.18";
	public static final String BK2_URN = "urn:oid:1.3.6.1.4.1.25484.494450.3";
	public static final String VORNAME_URN = "urn:oid:2.5.4.42";
	public static final String NACHNAME_URN = "urn:oid:2.5.4.4";
	public static final String VERTRAUENSNIVEAU_URN = "urn:oid:1.2.40.0.10.2.1.1.261.94";
	private static final String[] KNOWN_ATTRIBUTES = new String[] { POSTKORB_HANDLE_URN, BK2_URN, VORNAME_URN, NACHNAME_URN, VERTRAUENSNIVEAU_URN };

	static User map(Saml2Authentication authentication) {
		DefaultSaml2AuthenticatedPrincipal principal = (DefaultSaml2AuthenticatedPrincipal) authentication.getPrincipal();
		var token = RandomStringUtils.random(128, 0, 0, true, true, null, new SecureRandom());
		return new User.UserBuilder()
		  .id(principal.getName())
		  .username(principal.getName())
		  .firstName(getFirstName(principal))
		  .lastName(getLastName(principal))
		  .postkorbHandle(getPostkornHandle(principal))
		  .trustLevel(getTrustLevel(principal))
		  .samlToken(authentication.getSaml2Response())
		  .refreshCode(token)
		  .unknownAttributes(getUnknownAttributes(principal.getAttributes()))
		  .build();
	}

	static String getFirstName(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(VORNAME_URN);
	}

	static String getLastName(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(NACHNAME_URN);
	}

	static String getPostkornHandle(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(POSTKORB_HANDLE_URN);
	}

	static String getTrustLevel(DefaultSaml2AuthenticatedPrincipal principal) {
		return principal.getFirstAttribute(VERTRAUENSNIVEAU_URN);
	}

	static List<Map.Entry<String, List<Object>>> getUnknownAttributes(Map<String, List<Object>> attributes) {
		return attributes.entrySet().stream().filter(entry -> !ArrayUtils.contains(KNOWN_ATTRIBUTES, entry.getKey())).toList();
	}

}
