/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.proxy.AntragraumGrpcFileMetadata;

@ExtendWith(MockitoExtension.class)
class FileRemoteServiceTest {

	private static final FileIdentificationData FILE_IDENTIFICATION_DATA = FileIdentificationDataTestFactory.create();

	@Spy
	@InjectMocks
	private FileRemoteService remoteService;

	@Mock
	private FileRestClient restClient;

	@Mock
	private OzgFileMapper ozgFileMapper;

	@Nested
	class TestLoadingFile {

		@BeforeEach
		void init() {
			lenient().when(restClient.getFileMetadata(any(FileIdentificationData.class), anyString())).thenReturn(
			  AntragraumGrpcGetAttachmentMetadataResponseTestFactory.create());
			lenient().when(ozgFileMapper.fromGrpcFileMetadata(any(AntragraumGrpcFileMetadata.class), anyString()))
			  .thenReturn(OzgFileTestFactory.create());
		}

		@Test
		void shouldLoadFileMetadata() {
			var file = remoteService.getFile(FILE_IDENTIFICATION_DATA, CHANNEL_ADDRESS);

			assertThat(file).isNotNull();
		}

		@Test
		void shouldHaveFileName() {
			var file = remoteService.getFile(FILE_IDENTIFICATION_DATA, CHANNEL_ADDRESS);

			assertThat(file.fileName()).isEqualTo(FILE_NAME);
		}

		@Test
		void shouldHaveFileSize() {
			var file = remoteService.getFile(FILE_IDENTIFICATION_DATA, CHANNEL_ADDRESS);

			assertThat(file.fileSize()).isEqualTo(FILE_SIZE);
		}

		@Test
		void shouldHaveContentType() {
			var file = remoteService.getFile(FILE_IDENTIFICATION_DATA, CHANNEL_ADDRESS);

			assertThat(file.contentType()).isEqualTo(CONTENT_TYPE);
		}

		@Test
		void shouldHaveFileId() {
			var file = remoteService.getFile(FILE_IDENTIFICATION_DATA, CHANNEL_ADDRESS);

			assertThat(file.id()).isEqualTo(FILE_ID);
		}

		@Test
		void shouldHandleEmpty() {
			var response = AntragraumGrpcGetAttachmentMetadataResponseTestFactory.create();
			response.setFileMetadata(null);

			when(restClient.getFileMetadata(any(FileIdentificationData.class), anyString())).thenReturn(response);

			var file = remoteService.getFile(FILE_IDENTIFICATION_DATA, CHANNEL_ADDRESS);

			assertThat(file).isNull();
		}

	}

	@Nested
	class TestDownloadFileContent {

		private final ByteArrayOutputStream output = new ByteArrayOutputStream();

		@Test
		void shouldCallDownloadFileObserver() {
			remoteService.downloadFileContent(FILE_IDENTIFICATION_DATA, CHANNEL_ADDRESS, output);

			verify(restClient).downloadFileContent(eq(FILE_IDENTIFICATION_DATA), eq(CHANNEL_ADDRESS), any(OutputStream.class));
		}

	}

	@Nested
	class TestUploadFile {

		@Test
		void shouldCallRestClient() {
			remoteService.uploadFile(OzgUploadFileTestFactory.create(), CHANNEL_ADDRESS);

			verify(restClient).uploadFile(any(OzgUploadFile.class), eq(CHANNEL_ADDRESS), any());
		}

	}

}