/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.FileRestClient.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.client.match.MockRestRequestMatchers.*;
import static org.springframework.test.web.client.response.MockRestResponseCreators.*;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.client.RestClientTest;
import org.springframework.boot.test.web.client.MockServerRestClientCustomizer;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;
import org.springframework.test.web.client.MockRestServiceServer;
import org.springframework.web.client.RestClient;
import org.springframework.web.client.RestClientException;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.security.AuthenticationHelper;
import de.ozgcloud.antragsraum.security.UserTestFactory;

@SpringJUnitConfig(classes = { FileRestClientTestConfiguration.class })
@RestClientTest(FileRestClient.class)
class FileRestClientTest {

	private static final FileIdentificationData FILE_IDENTIFICATION_DATA = FileIdentificationDataTestFactory.create();
	static final String ADDRESS = "http://localhost:8382";

	private FileRestClient fileRestClient;

	@Autowired
	private RestClient.Builder restClientBuilder;

	@Autowired
	private MockServerRestClientCustomizer customizer;

	private MockRestServiceServer server;

	private final ObjectMapper objectMapper = new ObjectMapper();

	@BeforeEach
	void setupRestClient() {
		customizer.customize(restClientBuilder);
		server = customizer.getServer(restClientBuilder);
		fileRestClient = new FileRestClient(restClientBuilder.build());
	}

	@Nested
	class TestGetFileMetadata {

		private static final String FILE_METADATA_URI = ADDRESS + FIND_FILE_METADATA_API;

		@Test
		void shouldLoadFileMetadata() throws JsonProcessingException {
			try (var authenticationHelper = mockStatic(AuthenticationHelper.class)) {
				authenticationHelper.when(AuthenticationHelper::getSamlToken).thenReturn(UserTestFactory.SAML_TOKEN);
				server.expect(requestTo(FILE_METADATA_URI)).andRespond(
				  withSuccess(objectMapper.writeValueAsString(AntragraumGrpcGetAttachmentMetadataResponseTestFactory.create()),
					MediaType.APPLICATION_JSON));

				var fileMetadata = fileRestClient.getFileMetadata(FILE_IDENTIFICATION_DATA, ADDRESS + FIND_FILE_METADATA_API);

				assertThat(fileMetadata).isNotNull();
			}
		}

		@Test
		void shouldThrowRestClientException() {
			try (var authenticationHelper = mockStatic(AuthenticationHelper.class)) {
				authenticationHelper.when(AuthenticationHelper::getSamlToken).thenReturn(UserTestFactory.SAML_TOKEN);
				server.expect(requestTo(FILE_METADATA_URI)).andRespond(withStatus(HttpStatus.INTERNAL_SERVER_ERROR));

				assertThatExceptionOfType(RestClientException.class).isThrownBy(
				  () -> fileRestClient.getFileMetadata(FILE_IDENTIFICATION_DATA, ADDRESS + GET_FILE_CONTENT_API));
			}
		}

	}

	@Nested
	class TestUploadingFile {

		@Test
		void shouldUploadFile() throws JsonProcessingException {
			server.expect(requestTo(ADDRESS + UPLOAD_FILE_API))
			  .andRespond(withSuccess(objectMapper.writeValueAsString(UUID.randomUUID().toString()), MediaType.APPLICATION_JSON));

			CompletableFuture<String> fileId = new CompletableFuture<>();
			fileRestClient.uploadFile(OzgUploadFileTestFactory.create(), ADDRESS + UPLOAD_FILE_API, fileId);

			assertThat(fileId).isCompleted();
		}

		@Test
		void shouldHandleEmptyUploadFile() {
			server.expect(requestTo(ADDRESS + UPLOAD_FILE_API))
			  .andRespond(withSuccess("", MediaType.APPLICATION_JSON));

			CompletableFuture<String> fileId = new CompletableFuture<>();
			fileRestClient.uploadFile(OzgUploadFileTestFactory.create(), ADDRESS + UPLOAD_FILE_API, fileId);

			assertThat(fileId).withFailMessage(NO_FILE_ID_RECEIVED_MESSAGE).isCompletedExceptionally();
		}

	}

	@Nested
	class TestDownloadFileContent {

		private static final String FILE_CONTENT_URI = ADDRESS + GET_FILE_CONTENT_API;

		@Test
		void shouldDownloadFileContent() throws IOException {
			try (var authenticationHelper = mockStatic(AuthenticationHelper.class)) {
				authenticationHelper.when(AuthenticationHelper::getSamlToken).thenReturn(UserTestFactory.SAML_TOKEN);
				server.expect(requestTo(FILE_CONTENT_URI))
				  .andRespond(withSuccess(OzgUploadFileTestFactory.MULTIPART_FILE.getBytes(), MediaType.APPLICATION_OCTET_STREAM));

				var out = new ByteArrayOutputStream();
				fileRestClient.downloadFileContent(FILE_IDENTIFICATION_DATA, ADDRESS + GET_FILE_CONTENT_API, out);
				var res = out.toByteArray();

				assertThat(res).isEqualTo(OzgUploadFileTestFactory.MULTIPART_FILE.getBytes());
			}
		}

		@Test
		void shouldThrowTechnicalException() throws IOException {
			try (var authenticationHelper = mockStatic(AuthenticationHelper.class)) {
				authenticationHelper.when(AuthenticationHelper::getSamlToken).thenReturn(UserTestFactory.SAML_TOKEN);
				server.expect(requestTo(FILE_CONTENT_URI))
				  .andRespond(withSuccess(OzgUploadFileTestFactory.MULTIPART_FILE.getBytes(), MediaType.APPLICATION_OCTET_STREAM));

				var out = mock(ByteArrayOutputStream.class);
				doThrow(IOException.class).when(out).write(any(byte[].class), anyInt(), anyInt());

				assertThatExceptionOfType(TechnicalException.class).isThrownBy(
				  () -> fileRestClient.downloadFileContent(FILE_IDENTIFICATION_DATA, ADDRESS + GET_FILE_CONTENT_API, out));
			}
		}

		@Test
		void shouldThrowRestClientException() {
			try (var authenticationHelper = mockStatic(AuthenticationHelper.class)) {
				authenticationHelper.when(AuthenticationHelper::getSamlToken).thenReturn(UserTestFactory.SAML_TOKEN);
				server.expect(requestTo(FILE_CONTENT_URI)).andRespond(withStatus(HttpStatus.INTERNAL_SERVER_ERROR));

				assertThatExceptionOfType(RestClientException.class).isThrownBy(
				  () -> fileRestClient.downloadFileContent(FILE_IDENTIFICATION_DATA, ADDRESS + GET_FILE_CONTENT_API,
					mock(ByteArrayOutputStream.class)));
			}
		}

	}

}