/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.events;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.web.client.RestClient;
import org.springframework.web.client.RestClient.RequestHeadersUriSpec;
import org.springframework.web.client.RestClient.ResponseSpec;

import de.ozgcloud.antragsraum.proxy.InfoGrpcInformationByIdResponse;
import de.ozgcloud.antragsraum.proxy.InfoGrpcInformationResponse;
import de.ozgcloud.common.errorhandling.TechnicalException;

class NachrichtEventRestClientTest {

	@Spy
	@InjectMocks
	private NachrichtEventRestClient nachrichtClient;
	@Mock
	private RestClient client;

	@DisplayName("Get nachricht events by postfachId")
	@Nested
	class TestGetNachrichtEventsByPostfach {

		private final InfoGrpcInformationResponse grpcResponse = GrpcInformationResponseTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(grpcResponse).when(nachrichtClient).get(any(), any(), any());
		}

		@Test
		void shouldCallBuildGetRequest() {
			nachrichtClient.getNachrichtEventsByPostfach(NachrichtEventTestFactory.POSTFACH_ID);

			verify(nachrichtClient).get(NachrichtEventRestClient.INFORMATION_LIST_ENDPOINT, InfoGrpcInformationResponse.class,
			  NachrichtEventTestFactory.POSTFACH_ID);
		}

		@Test
		void shouldReturnResponse() {
			var response = nachrichtClient.getNachrichtEventsByPostfach(NachrichtEventTestFactory.POSTFACH_ID);

			assertThat(response).isEqualTo(grpcResponse);
		}

	}

	@DisplayName("Get nachricht event by id")
	@Nested
	class TestGetNachrichtEventById {

		private final InfoGrpcInformationByIdResponse grpcResponse = GrpcInformationByIdResponseFactory.create();

		@BeforeEach
		void init() {
			doReturn(grpcResponse).when(nachrichtClient).get(any(), any(), any());
		}

		@Test
		void shouldCallBuildGetRequest() {
			nachrichtClient.getNachrichtEventById(NachrichtEventTestFactory.ID);

			verify(nachrichtClient).get(NachrichtEventRestClient.INFORMATION_ENDPOINT, InfoGrpcInformationByIdResponse.class,
			  NachrichtEventTestFactory.ID);
		}

		@Test
		void shouldReturnResponse() {
			var response = nachrichtClient.getNachrichtEventById(NachrichtEventTestFactory.ID);

			assertThat(response).isEqualTo(grpcResponse);
		}

	}

	@DisplayName("Build get request")
	@Nested
	class TestBuildGetRequest {

		@Mock
		private ResponseSpec response;

		@Mock
		private RequestHeadersUriSpec request;

		@BeforeEach
		@SuppressWarnings("unchecked")
		void init() {
			when(client.get()).thenReturn(request);
			when(request.uri(any(String.class), any(Object.class))).thenReturn(request);
			when(request.retrieve()).thenThrow(new RuntimeException("dummyMessage"));
		}

		@DisplayName("should throw a TechnicalException if restClient causing errors")
		@Test
		void shouldThrowException() {
			assertThrows(TechnicalException.class, () -> nachrichtClient.get("dummyEndpoint", Object.class));
		}

	}

}
