/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static de.ozgcloud.antragsraum.nachricht.AntragraumproxyGrpcRueckfrageTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;

import org.apache.commons.io.FileUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.events.NachrichtEventTestFactory;
import lombok.extern.log4j.Log4j2;

@SpringBootTest
@SpringJUnitConfig(classes = { NachrichtenRemoteServiceTestConfiguration.class })
@Log4j2
class NachrichtenRemoteServiceITCase {

	public static final String ADDRESS = "localhost";

	String samlResponse;

	@SpyBean
	NachrichtenRemoteService nachrichtenRemoteService;

	@BeforeEach
	void setupSamlResponse() {
		try {
			samlResponse = FileUtils.readFileToString(new File("src/test/resources/testfiles/SamlResponse.xml"), Charset.defaultCharset());
		} catch (IOException e) {
			LOG.error("Error reading SAML response data.", e);
		}
	}

	@Nested
	class TestGetNachrichtHeaders {

		private final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.createBuilder().address(ADDRESS).build();

		@BeforeEach
		void setup() {
			initUserPasswordToken();
		}

		@Test
		void shouldLoadNachrichtHeaders() {
			try {
				var nachrichtHeaders = nachrichtenRemoteService.findRueckfrageHeads(nachrichtEvent);

				assertThat(nachrichtHeaders).isNotNull();
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}

		@Test
		void shouldHaveRueckfrageId() {
			var nachrichtHeaders = nachrichtenRemoteService.findRueckfrageHeads(nachrichtEvent);

			assertThat(nachrichtHeaders.getFirst().id()).isEqualTo(RUECKFRAGE_ID);
		}

	}

	@Nested
	class TestGetNachricht {

		private final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.createBuilder().address(ADDRESS).build();

		@BeforeEach
		void setup() {
			initUserPasswordToken();
		}

		@Test
		void shouldLoadNachricht() {
			var nachricht = nachrichtenRemoteService.getRueckfrage(RUECKFRAGE_ID, nachrichtEvent);

			assertThat(nachricht).isNotNull();
		}

		@Test
		void shouldHaveRueckfrageId() {
			var nachricht = nachrichtenRemoteService.getRueckfrage(RUECKFRAGE_ID, nachrichtEvent).orElseThrow();

			assertThat(nachricht.id()).isEqualTo(RUECKFRAGE_ID);
		}

	}

	@Nested
	class TestSendNachricht {

		@Test
		void shouldSendNachricht() {
			initUserPasswordToken();

			try {
				var res = nachrichtenRemoteService.sendAnswer(ReplyNachrichtTestFactory.create(), ADDRESS);

				assertThat(res).isNotNull();
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}

	}

	private void initUserPasswordToken() {
		UsernamePasswordAuthenticationToken authentication = mock(UsernamePasswordAuthenticationToken.class);
		SecurityContextHolder.getContext().setAuthentication(authentication);
	}

}
