/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static de.ozgcloud.antragsraum.nachricht.NachrichtenService.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.List;
import java.util.Optional;
import java.util.Set;

import jakarta.validation.ConstraintViolation;
import jakarta.validation.Validator;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.command.CommandReference;
import de.ozgcloud.antragsraum.command.CommandReferenceTestFactory;
import de.ozgcloud.antragsraum.common.NotAccessibleException;
import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.events.NachrichtEventService;
import de.ozgcloud.antragsraum.events.NachrichtEventTestFactory;

@ExtendWith(MockitoExtension.class)
class NachrichtenServiceTest {

	@Spy
	@InjectMocks
	private NachrichtenService nachrichtenService;
	@Mock
	private NachrichtEventService nachrichtEventService;
	@Mock
	private NachrichtenRemoteService nachrichtenRemoteService;
	@Mock
	private Validator validator;

	@Nested
	class TestGetRueckfrageHeadersOfPostfach {

		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventsOfPostfachId(anyString())).thenReturn(
			  List.of(NachrichtEventTestFactory.create()));
			when(nachrichtenRemoteService.findRueckfrageHeads(any())).thenReturn(List.of(NachrichtTestFactory.createNachrichtHeader()));
		}

		@Test
		void shouldReturnRueckfrageHeader() {
			var result = nachrichtenService.getRueckfrageHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			assertThat(result).hasSize(1);
		}

		@Test
		void shouldOnlyAddSelfLink() {
			when(nachrichtenRemoteService.findRueckfrageHeads(any())).thenReturn(
			  List.of(NachrichtTestFactory.createNachrichtHeaderBuilder().accessible(false).build()));

			var headerWithSelfLink = RueckfrageTestFactory.createRueckfrageHeader();
			headerWithSelfLink.add(linkTo(NachrichtenController.class).withSelfRel());

			var result = nachrichtenService.getRueckfrageHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			assertThat(result.getFirst().getLinks()).isEqualTo(headerWithSelfLink.getLinks());
		}

		@Test
		void shouldAddSelfAndRueckfrageLink() {
			var fullyLinkedHeader = RueckfrageTestFactory.createRueckfrageHeader();
			fullyLinkedHeader.add(linkTo(NachrichtenController.class).withSelfRel());
			fullyLinkedHeader.add(linkTo(methodOn(NachrichtenController.class).getRueckfrage(fullyLinkedHeader.getNachrichtHeader().id(),
			  fullyLinkedHeader.getNachrichtHeader().nachrichtEventId())).withRel(RUECKFRAGE_LINK_RELATIONSHIP_NAME));

			var result = nachrichtenService.getRueckfrageHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			assertThat(result.getFirst().getLinks()).isEqualTo(fullyLinkedHeader.getLinks());
		}

	}

	@Nested
	class TestGetRueckfrage {

		private final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.create();

		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventById(anyString())).thenReturn(nachrichtEvent);
		}

		@Test
		void shouldMapRueckfrage() {
			when(nachrichtenRemoteService.getRueckfrage(NachrichtTestFactory.ID, nachrichtEvent)).thenReturn(
			  Optional.of(NachrichtTestFactory.createNachricht()));

			var result = nachrichtenService.getRueckfrage(NachrichtTestFactory.ID, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(result).isNotNull();
		}

		@Test
		void shouldThrowNotAccessibleException() {
			when(nachrichtenRemoteService.getRueckfrage(NachrichtTestFactory.ID, nachrichtEvent)).thenReturn(
			  Optional.of(NachrichtTestFactory.createNachrichtBuilder().accessible(false).build()));

			assertThatExceptionOfType(NotAccessibleException.class).isThrownBy(
				() -> nachrichtenService.getRueckfrage(NachrichtTestFactory.ID, NachrichtTestFactory.NACHRICHT_EVENT_ID))
			  .withMessage("Current user does not have the required trust level for the Rueckfrage with id '" + NachrichtTestFactory.ID + "'.");
		}

	}

	@Nested
	class TestGetNachrichtHeadersOfPostfachWithNachrichtenEventAvailable {

		final NachrichtEvent event = NachrichtEventTestFactory.create();

		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventsOfPostfachId(anyString())).thenReturn(List.of(event));
			when(nachrichtenRemoteService.findRueckfrageHeads(any())).thenReturn(
			  List.of(NachrichtTestFactory.createNachrichtHeaderBuilder().postfachId(event.postfachId()).build()));
		}

		@Test
		void shouldCallNachrichtRemoteService() {
			nachrichtenService.getNachrichtHeadersOfPostfach(event.postfachId());

			verify(nachrichtenRemoteService, atMostOnce()).findRueckfrageHeads(any(NachrichtEvent.class));
		}

		@Test
		void shouldHaveNachrichtHeaders() {
			var nachrichtHeaders = nachrichtenService.getNachrichtHeadersOfPostfach(event.postfachId());

			assertThat(nachrichtHeaders).hasSize(1);
		}

		@Test
		void shouldHavePostfachId() {
			var nachrichtHeaders = nachrichtenService.getNachrichtHeadersOfPostfach(event.postfachId());

			assertThat(nachrichtHeaders.stream().findFirst()).isPresent().map(NachrichtHeader::postfachId).hasValue(event.postfachId());
		}

	}

	@Nested
	class TestGetNachrichtHeadersOfPostfachWithNoNachrichtenEventAvailable {

		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventsOfPostfachId(anyString())).thenReturn(List.of());
		}

		@Test
		void shouldCallNachrichtEventService() {
			var nachrichten = nachrichtenService.getNachrichtHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			assertThat(nachrichten).isEmpty();
		}

		@Test
		void shouldNotCallNachrichtRemoteService() {
			nachrichtenService.getNachrichtHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			verify(nachrichtenRemoteService, never()).findRueckfrageHeads(any(NachrichtEvent.class));
		}

	}

	@Nested
	class TestSendNachricht {

		ReplyNachricht msg = ReplyNachrichtTestFactory.create();

		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventById(anyString())).thenReturn(NachrichtEventTestFactory.create());
			when(nachrichtenRemoteService.sendAnswer(any(ReplyNachricht.class), anyString())).thenReturn(CommandReferenceTestFactory.create());
		}

		@Test
		void shouldCallRemoteService() {
			nachrichtenService.sendRueckfrageAnswer(msg);

			verify(nachrichtenRemoteService).sendAnswer(any(ReplyNachricht.class), anyString());
		}

		@Test
		void shouldReturnCommandReference() {
			var result = nachrichtenService.sendRueckfrageAnswer(msg);

			assertThat(result).isInstanceOf(CommandReference.class);
		}

	}

	@Nested
	class TestValidation {

		@Test
		@SuppressWarnings("unchecked")
		void shouldNotCallSendOnValidationError() {
			when(validator.validate(any(ReplyNachricht.class))).thenReturn(Set.of(mock(ConstraintViolation.class)));

			assertThatException().isThrownBy(() -> nachrichtenService.sendRueckfrageAnswer(ReplyNachrichtTestFactory.create()));
		}

	}

}