/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.security;

import static de.ozgcloud.antragsraum.security.BayernIdSaml2Extension.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;

import javax.xml.namespace.QName;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.opensaml.core.config.InitializationService;
import org.opensaml.core.xml.XMLObject;
import org.opensaml.core.xml.schema.XSAny;
import org.opensaml.saml.saml2.core.Extensions;

@ExtendWith(MockitoExtension.class)
class BayernIdSaml2ExtensionTest {

	private static final String URN_1_1 = "urn:1.1";
	private static final String URN_2_2 = "urn.2.2";
	private static final String ORGANIZATION_DISPLAY_NAME = "TestOrganizationDisplayName";

	@Mock
	private BayernIdProperties bayernIdProperties;
	private BayernIdSaml2Extension bayernIdSaml2Extension;

	@Nested
	class TestCreateMethodElement {

		private XMLObject authMethodObject;

		@BeforeEach
		void setUp() {
			bayernIdSaml2Extension = new BayernIdSaml2Extension(bayernIdProperties);
			authMethodObject = bayernIdSaml2Extension.createMethod("test", "true");
		}

		@Test
		void shouldHaveLocalPart() {
			assertThat(authMethodObject.getElementQName().getLocalPart()).isEqualTo("test");
		}

		@Test
		void shouldHaveNamespace() {
			assertThat(authMethodObject.getElementQName().getNamespaceURI()).isEqualTo(NAMESPACE_URI);
		}

		@Test
		void shouldHavePrefix() {
			assertThat(authMethodObject.getElementQName().getPrefix()).isEqualTo(NAMESPACE_PREFIX);
		}

		@Test
		void shouldHaveOneChild() {
			assertThat(authMethodObject.getOrderedChildren()).hasSize(1);
		}

		@Test
		void shouldHaveChildLocalPart() {
			var element = getChild(authMethodObject);
			assertThat(element.getElementQName().getLocalPart()).isEqualTo(ENABLED_LOCAL_NAME);
		}

		@Test
		void shouldHaveChildPrefix() {
			var element = getChild(authMethodObject);
			assertThat(element.getElementQName().getPrefix()).isEqualTo(NAMESPACE_PREFIX);
		}

		@Test
		void shouldHaveChildNamespace() {
			var element = getChild(authMethodObject);
			assertThat(element.getElementQName().getNamespaceURI()).isEqualTo(NAMESPACE_URI);
		}

		@Test
		void shouldHaveValue() {
			var element = getChild(authMethodObject);
			assertThat(element.getTextContent()).isEqualTo("true");
		}

		private XSAny getChild(XMLObject authMethodElement) {
			return (XSAny) authMethodElement.getOrderedChildren().getFirst();
		}

	}

	@Nested
	class TestCreateRequestedAttributes {

		XMLObject requestAttributes;

		@BeforeEach
		void setUp() throws Exception {
			InitializationService.initialize();
			when(bayernIdProperties.getRequestedAttributeUrns()).thenReturn(List.of(URN_1_1, URN_2_2));
			bayernIdSaml2Extension = new BayernIdSaml2Extension(bayernIdProperties);
			requestAttributes = bayernIdSaml2Extension.createRequestedAttributes();
		}

		@Test
		void shouldHaveLocalPart() {
			assertThat(requestAttributes.getElementQName().getLocalPart()).isEqualTo(REQUESTED_ATTRIBUTES_LOCAL_NAME);
		}

		@Test
		void shouldHaveNamespace() {
			assertThat(requestAttributes.getElementQName().getNamespaceURI()).isEqualTo(NAMESPACE_URI);
		}

		@Test
		void shouldHavePrefix() {
			assertThat(requestAttributes.getElementQName().getPrefix()).isEqualTo(NAMESPACE_PREFIX);
		}

		@Test
		void shouldHaveTwoAttributes() {
			assertThat(requestAttributes.getOrderedChildren()).hasSize(2);
		}

		@Test
		void shouldHaveAttributeUrn1() {
			assertThat(getAttribute(requestAttributes, URN_1_1)).isPresent();
		}

		@Test
		void shouldHaveAttributeUrn2() {
			assertThat(getAttribute(requestAttributes, URN_2_2)).isPresent();
		}

		private Optional<XMLObject> getAttribute(XMLObject requestAttributes, String name) {
			return requestAttributes.getOrderedChildren().stream().filter(child ->
			  ((XSAny) child).getUnknownAttributes().containsValue(name)
			).findFirst();
		}

	}

	@Nested
	class TestCreateDisplayInformation {

		XMLObject displayInformation;

		@BeforeEach
		void setUp() {
			when(bayernIdProperties.getOrganizationDisplayName()).thenReturn(ORGANIZATION_DISPLAY_NAME);
			bayernIdSaml2Extension = new BayernIdSaml2Extension(bayernIdProperties);
			displayInformation = bayernIdSaml2Extension.createDisplayInformation();
		}

		@Test
		void shouldHaveLocalPart() {
			assertThat(displayInformation.getElementQName().getLocalPart()).isEqualTo(DISPLAY_INFORMATION_LOCAL_NAME);
		}

		@Test
		void shouldHaveNamespace() {
			assertThat(displayInformation.getElementQName().getNamespaceURI()).isEqualTo(NAMESPACE_URI);
		}

		@Test
		void shouldHavePrefix() {
			assertThat(displayInformation.getElementQName().getPrefix()).isEqualTo(NAMESPACE_PREFIX);
		}

		@Test
		void shouldHaveOneChild() {
			assertThat(displayInformation.getOrderedChildren()).hasSize(1);
		}

		@Test
		void shouldHaveOneGrandChild() {
			assertThat(displayInformation.getOrderedChildren().getFirst().getOrderedChildren()).hasSize(1);
		}

		@Test
		void shouldHaveGrandChildLocalPart() {
			var element = getGrandChild(displayInformation);
			assertThat(element.getElementQName().getLocalPart()).isEqualTo(ORGANIZATION_DISPLAY_NAME_LOCAL_NAME);
		}

		@Test
		void shouldHaveGrandChildPrefix() {
			var element = getGrandChild(displayInformation);
			assertThat(element.getElementQName().getPrefix()).isEqualTo(CLASSIC_UI_PREFIX);
		}

		@Test
		void shouldHaveGrandChildNamespace() {
			var element = getGrandChild(displayInformation);
			assertThat(element.getElementQName().getNamespaceURI()).isEqualTo(CLASSIC_UI_URI);
		}

		@Test
		void shouldHaveGrandChildValue() {
			var element = getGrandChild(displayInformation);
			assertThat(element.getTextContent()).isEqualTo(ORGANIZATION_DISPLAY_NAME);
		}

		private XSAny getGrandChild(XMLObject authMethodElement) {
			return (XSAny) authMethodElement.getOrderedChildren().getFirst().getOrderedChildren().getFirst();
		}

	}

	@Nested
	class TestCreateExtension {

		Extensions extensions;

		@BeforeEach
		void setUp() throws Exception {
			InitializationService.initialize();
			when(bayernIdProperties.getRequestedAttributeUrns()).thenReturn(List.of(URN_1_1, URN_2_2));
			when(bayernIdProperties.getAuthnMethods()).thenReturn(Map.of("Username", "true", "SuperSecretThing", "false"));
			bayernIdSaml2Extension = new BayernIdSaml2Extension(bayernIdProperties);
			extensions = bayernIdSaml2Extension.createAkdbExtension();
		}

		@Test
		void shouldHaveAkdbElement() {
			var akdbExtension = extensions.getOrderedChildren().getFirst();

			assertThat(akdbExtension.getElementQName().getPrefix()).isEqualTo(NAMESPACE_PREFIX);
			assertThat(akdbExtension.getElementQName().getLocalPart()).isEqualTo(AUTHENTICATION_REQUEST_LOCAL_NAME);
			assertThat(akdbExtension.getElementQName().getPrefix()).isEqualTo(NAMESPACE_PREFIX);
			assertThat(akdbExtension.getElementQName().getNamespaceURI()).isEqualTo(NAMESPACE_URI);
		}

		@Test
		void shouldHaveAttributeVersion() {
			XSAny akdbExtension = (XSAny) extensions.getOrderedChildren().getFirst();

			assertThat(akdbExtension.getUnknownAttributes()).containsEntry(new QName(VERSION_LOCAL_PART), "2");
		}

		@Test
		void shouldHaveAuthMethods() {
			var authMethods = getChildElement(AUTHN_METHODS_LOCAL_NAME, extensions.getOrderedChildren().getFirst());

			assertThat(authMethods).isPresent();
		}

		@Test
		void shouldHaveRequestAttributes() {
			var requestAttributes = getChildElement(REQUESTED_ATTRIBUTES_LOCAL_NAME, extensions.getOrderedChildren().getFirst());

			assertThat(requestAttributes).isPresent();
		}

		@Test
		void shouldHaveDisplayInformation() {
			var displayInformation = getChildElement(DISPLAY_INFORMATION_LOCAL_NAME, extensions.getOrderedChildren().getFirst());

			assertThat(displayInformation).isPresent();
		}

		private Optional<XMLObject> getChildElement(String localName, XMLObject extensions) {
			return extensions.getOrderedChildren().stream().filter(child ->
			  child.getElementQName().getLocalPart().equals(localName)
			).findFirst();
		}

	}

}