/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.security;

import static org.assertj.core.api.Assertions.*;

import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.test.util.ReflectionTestUtils;

import com.google.common.cache.CacheBuilder;
import com.google.common.testing.FakeTicker;

import de.ozgcloud.antragsraum.common.NotFoundException;

@ExtendWith(MockitoExtension.class)
class InMemoryUserDetailServiceTest {

	private final InMemoryUserDetailService userDetailService = new InMemoryUserDetailService();
	private final static String TEMP_ID = UUID.randomUUID().toString();
	private User user;

	@Nested
	class TestAddingUser {

		@BeforeEach
		void setup() {
			ReflectionTestUtils.setField(userDetailService, "jwtExpirationMinutes", 30L);
			user = UserTestFactory.create();
			userDetailService.tmpTokenUserIdCache = CacheBuilder.newBuilder()
			  .expireAfterAccess(10, TimeUnit.SECONDS)
			  .build();
			userDetailService.addUser(TEMP_ID, user);
		}

		@Test
		void shouldAddUser() {
			assertThat(userDetailService.getUser(UserTestFactory.USER_ID)).isNotNull();
		}

		@Test
		void shouldSetExpiryTimeDateWhenAddUser() {
			var date = ZonedDateTime.now(ZoneId.of("UTC")).plusMinutes(30L).toInstant();
			assertThat(userDetailService.getUser(UserTestFactory.USER_ID).getTokenExpiresAt()).isCloseTo(date, 10);
		}

		@Test
		void shouldGetUserByCode() {
			assertThat(userDetailService.getUserByCode(TEMP_ID)).isNotNull();
		}

		@Test
		void shouldThrowNotFoundExceptionWrongTempId() {
			assertThatExceptionOfType(NotFoundException.class).isThrownBy(() -> userDetailService.getUserByCode("OTHER"));
		}

		@Test
		void shouldLoadUser() {
			userDetailService.addUser(TEMP_ID, user);

			assertThat(userDetailService.loadUserByUsername(UserTestFactory.USER_NAME)).isNotNull();
		}

	}

	@Nested
	class TestSetUser {

		@BeforeEach
		void setup() {
			ReflectionTestUtils.setField(userDetailService, "jwtExpirationMinutes", 30L);
			user = UserTestFactory.create();
			userDetailService.tmpTokenUserIdCache = CacheBuilder.newBuilder()
			  .expireAfterAccess(10, TimeUnit.SECONDS)
			  .build();
			userDetailService.setUser(TEMP_ID, user);
		}

		@Test
		void shouldSetUser() {
			assertThat(userDetailService.getUser(UserTestFactory.USER_ID)).isNotNull();
		}

		@Test
		void shouldNotSetExpiryTimeDateWhenSetUser() {
			assertThat(userDetailService.getUser(UserTestFactory.USER_ID).getTokenExpiresAt()).isEqualTo(user.getTokenExpiresAt());
		}

	}

	@Nested
	class TestExpiringCodeCache {

		FakeTicker ticker = new FakeTicker();

		@BeforeEach
		void setup() {
			ReflectionTestUtils.setField(userDetailService, "jwtExpirationMinutes", 30L);

			user = UserTestFactory.create();
			userDetailService.tmpTokenUserIdCache = CacheBuilder.newBuilder()
			  .expireAfterAccess(1, TimeUnit.SECONDS)
			  .ticker(ticker)
			  .build();
			userDetailService.addUser(TEMP_ID, user);

			userDetailService.getUserByCode(TEMP_ID);
			ticker.advance(2, TimeUnit.SECONDS);
		}

		@Test
		void shouldThrowExceptionWhenGetUserByCode() {
			assertThatExceptionOfType(NotFoundException.class).isThrownBy(() -> userDetailService.getUserByCode(TEMP_ID));
		}

	}

	@Nested
	class TestUninitializedCache {

		@Test
		void shouldThrowExceptionWhenGetUserByCode() {
			assertThatExceptionOfType(NotFoundException.class).isThrownBy(() -> userDetailService.getUserByCode(TEMP_ID));
		}

	}

	@Nested
	class TestLoadingSamlToken {

		@BeforeEach
		void setup() {
			user = UserTestFactory.create();

			ReflectionTestUtils.setField(userDetailService, "jwtExpirationMinutes", 30L);
			userDetailService.addUser(TEMP_ID, user);
		}

		@Test
		void shouldReturnToken() {
			assertThat(userDetailService.getSamlTokenOfUser(user.getId())).isEqualTo(UserTestFactory.SAML_TOKEN);
		}

	}

	@Nested
	class TestHandlingRefreshToken {

		@BeforeEach
		void setup() {
			user = UserTestFactory.create();
			ReflectionTestUtils.setField(userDetailService, "jwtExpirationMinutes", 30L);
			userDetailService.addUser(UUID.randomUUID().toString(), user);
		}

		@Test
		void shouldGetUser() {
			var updatedUser = userDetailService.getUser(new AuthCode(user.getRefreshCode()));

			assertThat(updatedUser).isPresent();
		}

		@Test
		void shouldNotGetUserBecauseUnknownRefreshCode() {
			var user = userDetailService.getUser(new AuthCode(UUID.randomUUID().toString()));

			assertThat(user).isNotPresent();
		}

		@Test
		void shouldNotGetUserBecauseRefreshCodeExpired() {
			var localUser = UserTestFactory.createBuilder()
			  .tokenExpiresAt(Date.from(ZonedDateTime.now(ZoneId.of("UTC")).minusMinutes(31L).toInstant()))
			  .build();
			userDetailService.setUser(UUID.randomUUID().toString(), localUser);

			var userOptional = userDetailService.getUser(new AuthCode(localUser.getRefreshCode()));

			assertThat(userOptional).isNotPresent();
		}

		@Test
		void shouldUpdateRefreshCode() {
			var user = UserTestFactory.create();
			userDetailService.updateRefreshCodeOf(user);

			var updatedUser = userDetailService.getUser(user.getId());
			assertThat(updatedUser.getRefreshCode()).isNotEqualTo(user.getRefreshCode());
		}

	}

	@Nested
	class TestUserMapCleanUp {

		private User expiredUser;
		private User validUser;

		@BeforeEach
		void setup() {
			ReflectionTestUtils.setField(userDetailService, "jwtExpirationMinutes", 30L);
			validUser = UserTestFactory.create();
			userDetailService.setUser(UUID.randomUUID().toString(), validUser);
			expiredUser = UserTestFactory.createBuilder()
			  .id(UUID.randomUUID().toString())
			  .tokenExpiresAt(Date.from(ZonedDateTime.now(ZoneId.of("UTC")).minusMinutes(5L).toInstant())).build();
			userDetailService.setUser(UUID.randomUUID().toString(), expiredUser);
		}

		@Test
		void shouldRemoveExpiredUser() {
			userDetailService.userCleanUp();

			assertThat(userDetailService.getUser(expiredUser.getId())).isNull();
		}

		@Test
		void shouldNotRemoveValidUser() {
			userDetailService.userCleanUp();

			assertThat(userDetailService.getUser(validUser.getId())).isNotNull();
		}

		@Test
		void shouldRemoveOnLogout() {
			userDetailService.logout(validUser);

			assertThat(userDetailService.getUser(validUser.getId())).isNull();
		}

	}

}