/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.security;

import static de.ozgcloud.antragsraum.security.JwtTokenProvider.*;
import static org.assertj.core.api.Assertions.*;

import java.time.ZonedDateTime;
import java.util.Date;
import java.util.UUID;

import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.test.util.ReflectionTestUtils;

import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.MalformedJwtException;
import io.jsonwebtoken.security.Keys;
import io.jsonwebtoken.security.SignatureException;

class JwtTokenVerifierTest {

	private static final String SECRET = RandomStringUtils.random(256);
	JwtTokenVerifier verifier = new JwtTokenVerifier();

	@BeforeEach
	void setup() {
		ReflectionTestUtils.setField(verifier, "jwtSecret", SECRET);
	}

	@Nested
	class TestJwtValidation {

		@Test
		void shouldValidate() {
			var token = createToken(SECRET.getBytes(), Date.from(ZonedDateTime.now().plusMinutes(10).toInstant()), UserTestFactory.USER_ID);

			assertThat(verifier.validate(token)).isPresent();
		}

		@Test
		void shouldThrowMalformedJwtException() {
			var token = "sdkjfsdlkfsjkhlf";

			verifier.validate(token).ifPresent(jwtParser ->
			  assertThatExceptionOfType(MalformedJwtException.class).isThrownBy(() -> jwtParser.parse(token)).isNotNull()
			);
		}

		@Test
		void shouldThrowSignatureException() {
			var otherSecret = RandomStringUtils.random(256);
			var token = createToken(otherSecret.getBytes(), Date.from(ZonedDateTime.now().plusMinutes(10).toInstant()), "");

			verifier.validate(token).ifPresent(jwtParser ->
			  assertThatExceptionOfType(SignatureException.class).isThrownBy(() -> jwtParser.parse(token)).isNotNull()
			);
		}

		@Test
		void shouldThrowExpiredJwtException() {
			var token = createToken(SECRET.getBytes(), Date.from(ZonedDateTime.now().minusNanos(10).toInstant()), "");

			verifier.validate(token).ifPresent(jwtParser ->
			  assertThatExceptionOfType(ExpiredJwtException.class).isThrownBy(() -> jwtParser.parse(token)).isNotNull()
			);
		}

		@Test
		void shouldGetClaims() {
			var token = createToken(SECRET.getBytes(), Date.from(ZonedDateTime.now().plusMinutes(10).toInstant()), UserTestFactory.USER_ID);
			var parserOptional = verifier.validate(token);

			assertThat(parserOptional).isPresent().map(jwtParser -> assertThatObject(verifier.getJws(jwtParser, token)).isNotNull());
		}

		private String createToken(byte[] signingKey, Date expiration, String userName) {
			return Jwts.builder()
			  .header().add("typ", TOKEN_TYPE)
			  .and()
			  .signWith(Keys.hmacShaKeyFor(signingKey), Jwts.SIG.HS512)
			  .expiration(expiration)
			  .issuedAt(Date.from(ZonedDateTime.now().toInstant()))
			  .id(UUID.randomUUID().toString())
			  .issuer(TOKEN_ISSUER)
			  .audience().add(TOKEN_AUDIENCE)
			  .and()
			  .subject(userName)
			  .claim("rol", null)
			  .claim("firstname", UserTestFactory.FIRST_NAME)
			  .claim("lastname", UserTestFactory.LAST_NAME)
			  .claim("postkorbhandle", UserTestFactory.POSTKORB_HANDLE)
			  .claim("trustlevel", UserTestFactory.TRUST_LEVEL)
			  .compact();
		}

	}

}